# -*- encoding: utf-8 -*-
# stub: ame 1.0.1 ruby lib

Gem::Specification.new do |s|
  s.name = "ame".freeze
  s.version = "1.0.1"

  s.required_rubygems_version = Gem::Requirement.new(">= 0".freeze) if s.respond_to? :required_rubygems_version=
  s.require_paths = ["lib".freeze]
  s.authors = ["Nikolai Weibull".freeze]
  s.date = "2015-10-31"
  s.description = "                                      Ame\n\n  Ame provides a simple command-line interface API for Ruby\u00B9.  It can be used\n  to provide both simple interfaces like that of \u2039rm\u203A\u00B2 and complex ones like\n  that of \u2039git\u203A\u00B3.  It uses Ruby\u2019s own classes, methods, and argument lists to\n  provide an interface that is both simple to use from the command-line side\n  and from the Ruby side.  The provided command-line interface is flexible and\n  follows commond standards for command-line processing.\n\n\u00B9 See http://ruby-lang.org/\n\u00B2 See http://pubs.opengroup.org/onlinepubs/9699919799/utilities/rm.html\n\u00B3 See http://git-scm.com/docs/\n\n\u00A7 Usage\n\n    Let\u2019s begin by looking at two examples, one where we mimic the POSIX\u00B9\n    command-line interface to the \u2039rm\u203A command.  Looking at the entry\u00B2 in the\n    standard, \u2039rm\u203A takes the following options:\n\n  = -f. = Do not prompt for confirmation.\n  = -i. = Prompt for confirmation.\n  = -R. = Remove file hierarchies.\n  = -r. = Equivalent to /-r/.\n\n    It also takes the following arguments:\n\n  = FILE. = A pathname or directory entry to be removed.\n\n    And actually allows one or more of these /FILE/ arguments to be given.\n\n    We also note that the \u2039rm\u203A command is described as a command to \u201Cremove\n    directory entries\u201D.\n\n  \u00B9 See http://pubs.opengroup.org/onlinepubs/9699919799/utilities/contents.html\n  \u00B2 See http://pubs.opengroup.org/onlinepubs/9699919799/utilities/rm.html\n\n    Let\u2019s turn this specification into one using Ame\u2019s API.  We begin by adding\n    a flag for each of the options listed above:\n\n      class Rm < Ame::Root\n        flag 'f', '', false, 'Do not prompt for confirmation'\n        flag 'i', '', nil, 'Prompt for confirmation' do |options|\n          options['f'] = false\n        end\n        flag 'R', '', false, 'Remove file hierarchies'\n        flag 'r', '', nil, 'Equivalent to -R' do |options|\n          options['r'] = true\n        end\n\n    A flag\u00B9 is a boolean option that doesn\u2019t take an argument.  Each flag gets\n    a short and long name, where an empty name means that there\u2019s no\n    corresponding short or long name for the flag, a default value (true,\n    false, or nil), and a description of what the flag does.  Each flag can\n    also optionally take a block that can do further processing.  In this case\n    we use this block to modify the Hash that maps option names to their values\n    passed to the block to set other flags\u2019 values than the ones that the block\n    is associated with.  As these flags (\u2018i\u2019 and \u2018r\u2019) aren\u2019t themselves of\n    interest, their default values have been set to nil, which means that they\n    won\u2019t be included in the Hash that maps option names to their values when\n    passed to the method.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#flag-class-method\n\n    There are quite a few other kinds of options besides flags that can be\n    defined using Ame, but flags are all that are required for this example.\n    We\u2019ll get to the other kinds in later examples.\n\n    Next we add a \u201Csplus\u201D argument.\n\n        splus 'FILE', String, 'File to remove'\n\n    A splus\u00B9 argument is like a Ruby \u201Csplat\u201D, that is, an Array argument at the\n    end of the argument list to a method preceded by a star, except that a\n    splus requires at least one argument.  A splus argument gets a name for the\n    argument (\u2039FILE\u203A), the type of argument it represents (String), and a\n    description.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#splus-class-method\n\n    Then we add a description of the command (method) itself:\n\n        description 'Remove directory entries'\n\n    Descriptions\u00B9 will be used in help output to assist the user in using the\n    command.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#description-class-method\n\n    Finally, we add the Ruby method that\u2019ll implement the command (all\n    preceding code included here for completeness):\n\n      class Rm < Ame::Root\n        version '1.0.0'\n\n        flag 'f', '', false, 'Do not prompt for confirmation'\n        flag 'i', '', nil, 'Prompt for confirmation' do |options|\n          options['f'] = false\n        end\n        flag 'R', '', false, 'Remove file hierarchies'\n        flag 'r', '', nil, 'Equivalent to -R' do |options|\n          options['r'] = true\n        end\n        splus 'FILE', String, 'File to remove'\n        description 'Remove directory entries'\n        def rm(files, options = {})\n          require 'fileutils'\n          FileUtils.send options['R'] ? :rm_r : :rm,\n            [first] + rest, :force => options['f']\n        end\n      end\n\n    Actually, another bit of code was also added, namely\n\n        version '1.0.0'\n\n    This sets the version\u00B9 String of the command.  This information is used\n    when the command is invoked with the \u201C\u2039--version\u203A\u201D flag.  This flag is\n    automatically added, so you don\u2019t need to add it yourself.  Another flag,\n    \u201C\u2039--help\u203A\u201D, is also added automatically.  When given, this flag\u2019ll make Ame\n    output usage information of the command.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#version-class-method\n\n    To actually run the command, all you need to do is invoke\n\n      Rm.process\n\n    This\u2019ll invoke the command using the command-line arguments stored in\n    \u2039ARGV\u203A, but you can also specify other ones if you want to:\n\n      Rm.process 'rm', %w[-r /tmp/*]\n\n    The first argument to #process\u00B9 is the name of the method to invoke, which\n    defaults to \u2039File.basename($0)\u203A, and the second argument is an Array of\n    Strings that should be processed as command-line arguments passed to the\n    command.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#process-class-method\n\n    If you\u2019d store the complete \u2039Rm\u203A class defined above in a file called \u2039rm\u203A\n    and add \u2039#! /usr/bin/ruby -w\u203A at the beginning and \u2039Rm.process\u203A at the end,\n    you\u2019d have a fully functional \u2039rm\u203A command (after making it executable).\n    Let\u2019s see it in action:\n\n      % rm --help\n      Usage: rm [OPTIONS]... FILE...\n        Remove directory entries\n\n      Arguments:\n        FILE...  File to remove\n\n      Options:\n        -R             Remove file hierarchies\n        -f             Do not prompt for confirmation\n            --help     Display help for this method\n        -i             Prompt for confirmation\n        -r             Equivalent to -R\n            --version  Display version information\n      % rm --version\n      rm 1.0.0\n\n    Some commands are more complex than \u2039rm\u203A.  For example, \u2039git\u203A\u00B9 has a rather\n    complex command-line interface.  We won\u2019t mimic it all here, but let\u2019s\n    introduce the rest of the Ame API using a fake \u2039git\u203A clone as an example.\n\n  \u00B9 See http://git-scm.com/docs/\n\n    \u2039Git\u203A uses sub-commands to achieve most things.  Implementing sub-commands\n    with Ame is done using a \u201Cdispatch\u201D.  We\u2019ll discuss dispatches in more\n    detail later, but suffice it to say that a dispatch delegates processing to\n    a child class that\u2019ll handle the sub-command in question.  We begin by\n    defining our main \u2039git\u203A command using a class called \u2039Git\u203A under the\n    \u2039Git::CLI\u203A namespace:\n\n      module Git end\n      class Git::CLI < Ame::Root\n        version '1.0.0'\n        class Git < Ame::Class\n          description 'The stupid content tracker'\n          def initialize; end\n\n    We\u2019re setting things up to use the \u2039Git\u203A class as a dispatch in the\n    \u2039Git::CLI\u203A class.  The description on the \u2039initialize\u203A method will be used\n    as a description of the \u2039git\u203A dispatch command itself.\n\n    Next, let\u2019s add the \u2039format-patch\u203A\u00B9 sub-command:\n\n      description 'Prepare patches for e-mail submission'\n      flag   ?n, 'numbered', false, 'Name output in [PATCH n/m] format'\n      flag   ?N, 'no-numbered', nil,\n        'Name output in [PATCH] format' do |options|\n        options['numbered'] = false\n      end\n      toggle ?s, 'signoff', false,\n        'Add Signed-off-by: line to the commit message'\n      switch '', 'thread', 'STYLE', nil,\n        Ame::Types::Enumeration[:shallow, :deep],\n      'Controls addition of In-Reply-To and References headers'\n      flag   '', 'no-thread', nil,\n        'Disables addition of In-Reply-To and Reference headers' do |options, _|\n         options.delete 'thread'\n      end\n      option '', 'start-number', 'N', 1,\n        'Start numbering the patches at N instead of 1'\n      multioption '', 'to', 'ADDRESS', String,\n        'Add a To: header to the email headers'\n      optional 'SINCE', 'N/A', 'Generate patches for commits after SINCE'\n      def format_patch(since = '', options = {})\n        p since, options\n      end\n\n  \u00B9 See http://git-scm.com/docs/git-format-patch/\n\n    We\u2019re using quite a few new Ame commands here.  Let\u2019s look at each in turn:\n\n      toggle ?s, 'signoff', false,\n        'Add Signed-off-by: line to the commit message'\n\n    A \u201Ctoggle\u201D\u00B9 is a flag that also has an inverse.  Beyond the flags \u2018s\u2019 and\n    \u201Csignoff\u201D, the toggle also defines \u201Cno-signoff\u201D, which will set \u201Csignoff\u201D\n    to false.  This is useful if you want to support configuration files that\n    set \u201Csignoff\u201D\u2019s default to true, but still allow it to be overridden on the\n    command line.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#toggle-class-method\n\n    When using the short form of a toggle (and flag and switch), multiple ones\n    may be juxtaposed after the initial one.  For example, \u201C\u2039-sn\u203A\u201D is\n    equivalent to \u201C\u2039-s -n\u203A\u201D to \u201Cgit format-patch\u203A\u201D.\n\n      switch '', 'thread', 'STYLE', nil,\n        Ame::Types::Enumeration[:shallow, :deep],\n        'Controls addition of In-Reply-To and References headers'\n\n    A \u201Cswitch\u201D\u00B9 is an option that takes an optional argument.  This allows you\n    to have separate defaults for when the switch isn\u2019t present on the command\n    line and for when it\u2019s given without an argument.  The third argument to a\n    switch is the name of the argument.  We\u2019re also introducing a new concept\n    here in \u2039Ame::Types::Enumeration\u203A.  An enumeration\u00B2 allows you to limit the\n    allowed input to a set of Symbols.  An enumeration also has a default value\n    in the first item to its constructor (which is aliased as \u2039.[]\u203A).  In this\n    case, the \u201Cthread\u201D switch defaults to nil, but, when given, will default to\n    \u2039:shallow\u203A if no argument is given.  If an argument is given it must be\n    either \u201Cshallow\u201D or \u201Cdeep\u201D.  A switch isn\u2019t required to take an enumeration\n    as its argument default and can take any kind of default value for its\n    argument that Ame knows how to handle.  We\u2019ll look at this in more detail\n    later, but know that the type of the default value will be used to inform\n    Ame how to parse a command-line argument into a Ruby value.\n\n    An argument to a switch must be given, in this case, as \u201C\u2039--thread=deep\u203A\u201D\n    on the command line.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#switch-class-method\n  \u00B2 See http://disu.se/software/ame-1.0/api/user/Ame/Types/Enumeration/\n\n      option '', 'start-number', 'N', 1,\n        'Start numbering the patches at N instead of 1'\n\n    An \u201Coption\u201D\u00B9 is an option that takes an argument.  The argument must always\n    be present and may be given, in this case, as \u201C\u2039--start-number=2\u203A\u201D or\n    \u201C\u2039--start-number 2\u203A\u201D on the command line.  For a short-form option,\n    anything that follows the option is seen as an argument, so assuming that\n    \u201Cstart-number\u201D also had a short name of \u2018S\u2019, \u201C\u2039-S2\u203A\u201D would be equivalent to\n    \u201C\u2039-S 2\u203A\u201D, which would be equivalent to \u201C\u2039--start-number 2\u203A\u201D.  Note that\n    \u201C\u2039-snS2\u203A\u201D would still work as expected.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#option-class-method\n\n      multioption '', 'to', 'ADDRESS', String,\n        'Add a To: header to the email headers'\n\n    A \u201Cmultioption\u201D\u00B9 is an option that takes an argument and may be repeated\n    any number of times. Each argument will be added to an Array stored in the\n    Hash that maps option names to their values.  Instead of taking a default\n    argument, it takes a type for the argument (String, in this case).  Again,\n    types are used to inform Ame how to parse command-line arguments into Ruby\n    values.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#multioption-class-method\n\n      optional 'SINCE', 'N/A', 'Generate patches for commits after SINCE'\n\n    An \u201Coptional\u201D\u00B9 argument is an argument that isn\u2019t required.  If it\u2019s not\n    present on the command line it\u2019ll get its default value (the String\n    \u2039'N/A'\u203A, in this case).\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#optional-class-method\n\n    We\u2019ve now covered all kinds of options and one new kind of argument.  There\n    are three more types of argument (one that we\u2019ve already seen and two new)\n    that we\u2019ll look into now: \u201Cargument\u201D, \u201Csplat\u201D, and \u201Csplus\u201D.\n\n      description 'Annotate file lines with commit information'\n      argument 'FILE', String, 'File to annotate'\n      def annotate(file)\n        p file\n      end\n\n    An \u201Cargument\u201D\u00B9 is an argument that\u2019s required.  If it\u2019s not present on the\n    command line, an error will be raised (and by default reported to the\n    terminal).  As it\u2019s required, it doesn\u2019t take a default, but rather a type.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#argument-class-method\n\n      description 'Add file contents to the index'\n      splat 'PATHSPEC', String, 'Files to add content from'\n      def add(paths)\n        p paths\n      end\n\n    A \u201Csplat\u201D\u00B9 is an argument that\u2019s not required, but may be given any number\n    of times.  The type of a splat is the type of one argument and the type of\n    a splat as a whole is an Array of values of that type.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#splat-class-method\n\n      description 'Display gitattributes information'\n      splus 'PATHNAME', String, 'Files to list attributes of'\n      def check_attr(paths)\n        p paths\n      end\n\n    A \u201Csplus\u201D\u00B9 is an argument that\u2019s required, but may also be given any number\n    of times.  The type of a splus is the type of one argument and the type of\n    a splus as a whole is an Array of values of that type.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#splus-class-method\n\n    Now that we\u2019ve seen all kinds of options and arguments, let\u2019s look on an\n    additional tool at our disposal, the dispatch\u00B9.\n\n      class Remote < Ame::Class\n        description 'Manage set of remote repositories'\n        def initialize; end\n\n        description 'Shows a list of existing remotes'\n        flag 'v', 'verbose', false, 'Show remote URL after name'\n        def list(options = {})\n          p options\n        end\n\n        description 'Adds a remote named NAME for the repository at URL'\n        argument 'name', String, 'Name of the remote to add'\n        argument 'url', String, 'URL to the repository of the remote to add'\n        def add(name, url)\n          p name, url\n        end\n      end\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/Class#dispatch-class-method\n\n    Here we\u2019re defining a child class to Git::CLI::Git called \u201CRemote\u201D that\n    doesn\u2019t introduce anything new.  Then we set up the dispatch:\n\n      dispatch Remote, :default => 'list'\n\n    This adds a method called \u201Cremote\u201D to Git::CLI::Git that will dispatch\n    processing of the command line to an instance of the Remote class when\n    \u201C\u2039git remote\u203A\u201D is seen on the command line.  The \u201Cremote\u201D method expects an\n    argument that\u2019ll be used to decide what sub-command to execute.  Here we\u2019ve\n    specified that in the absence of such an argument, the \u201Clist\u201D method should\n    be invoked.\n\n    We add the same kind of dispatch to Git under Git::CLI:\n\n      dispatch Git\n\n    and then we\u2019re done.  Here\u2019s all the previous code in its entirety:\n\n      module Git end\n      class Git::CLI < Ame::Root\n        version '1.0.0'\n        class Git < Ame::Class\n          description 'The stupid content tracker'\n          def initialize; end\n\n          description 'Prepare patches for e-mail submission'\n          flag   ?n, 'numbered', false, 'Name output in [PATCH n/m] format'\n          flag   ?N, 'no-numbered', nil,\n            'Name output in [PATCH] format' do |options|\n            options['numbered'] = false\n          end\n          toggle ?s, 'signoff', false,\n            'Add Signed-off-by: line to the commit message'\n          switch '', 'thread', 'STYLE', nil,\n            Ame::Types::Enumeration[:shallow, :deep],\n            'Controls addition of In-Reply-To and References headers'\n          flag   '', 'no-thread', nil,\n            'Disables addition of In-Reply-To and Reference headers' do |options, _|\n             options.delete 'thread'\n          end\n          option '', 'start-number', 'N', 1,\n            'Start numbering the patches at N instead of 1'\n          multioption '', 'to', 'ADDRESS', String,\n            'Add a To: header to the email headers'\n          optional 'SINCE', 'N/A', 'Generate patches for commits after SINCE'\n          def format_patch(since = '', options = {})\n            p since, options\n          end\n\n          description 'Annotate file lines with commit information'\n          argument 'FILE', String, 'File to annotate'\n          def annotate(file)\n            p file\n          end\n\n          description 'Add file contents to the index'\n          splat 'PATHSPEC', String, 'Files to add content from'\n          def add(paths)\n            p paths\n          end\n\n          description 'Display gitattributes information'\n          splus 'PATHNAME', String, 'Files to list attributes of'\n          def check_attr(paths)\n            p paths\n          end\n\n          class Remote < Ame::Class\n            description 'Manage set of remote repositories'\n            def initialize; end\n\n            description 'Shows a list of existing remotes'\n            flag 'v', 'verbose', false, 'Show remote URL after name'\n            def list(options = {})\n              p options\n            end\n\n            description 'Adds a remote named NAME for the repository at URL'\n            argument 'name', String, 'Name of the remote to add'\n            argument 'url', String, 'URL to the repository of the remote to add'\n            def add(name, url)\n              p name, url\n            end\n          end\n          dispatch Remote, :default => 'list'\n        end\n        dispatch Git\n      end\n\n    If we put this code in a file called \u201Cgit\u201D and add \u2039#! /usr/bin/ruby -w\u203A at\n    the beginning and \u2039Git::CLI.process\u203A at the end, you\u2019ll have a very\n    incomplete git command-line interface on your hands.  Let\u2019s look at what\n    some of its \u2039--help\u203A output looks like:\n\n      % git --help\n      Usage: git [OPTIONS]... METHOD [ARGUMENTS]...\n      The stupid content tracker\n\n      Arguments:\n        METHOD          Method to run\n        [ARGUMENTS]...  Arguments to pass to METHOD\n\n      Options:\n        --help     Display help for this method\n        --version  Display version information\n\n      Methods:\n        add           Add file contents to the index\n        annotate      Annotate file lines with commit information\n        check-attr    Display gitattributes information\n        format-patch  Prepare patches for e-mail submission\n        remote        Manage set of remote repositories\n      % git format-patch --help\n      Usage: git format-patch [OPTIONS]... [SINCE]\n      Prepare patches for e-mail submission\n\n      Arguments:\n        [SINCE=N/A]  Generate patches for commits after SINCE\n\n      Options:\n        -N, --no-numbered     Name output in [PATCH] format\n            --help            Display help for this method\n        -n, --numbered        Name output in [PATCH n/m] format\n            --no-thread       Disables addition of In-Reply-To and Reference headers\n        -s, --signoff         Add Signed-off-by: line to the commit message\n            --start-number=N  Start numbering the patches at N instead of 1\n            --thread[=STYLE]  Controls addition of In-Reply-To and References headers\n            --to=ADDRESS*     Add a To: header to the email headers\n      % git remote --help\n      Usage: git remote [OPTIONS]... [METHOD] [ARGUMENTS]...\n      Manage set of remote repositories\n\n      Arguments:\n        [METHOD=list]   Method to run\n        [ARGUMENTS]...  Arguments to pass to METHOD\n\n      Options:\n        --help  Display help for this method\n\n      Methods:\n        add   Adds a remote named NAME for the repository at URL\n        list  Shows a list of existing remotes\n\n\u00A7 API\n\n    The previous section gave an introduction to the whole user API in an\n    informal and introductory way.  For an indepth reference to the user API,\n    see the {user API documentation}\u00B9.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/user/Ame/\n\n    If you want to extend the API or use it in some way other than as a\n    command-line-interface writer, see the {developer API documentation}\u00B9.\n\n  \u00B9 See http://disu.se/software/ame-1.0/api/developer/Ame/\n\n\u00A7 Financing\n\n    Currently, most of my time is spent at my day job and in my rather busy\n    private life.  Please motivate me to spend time on this piece of software\n    by donating some of your money to this project.  Yeah, I realize that\n    requesting money to develop software is a bit, well, capitalistic of me.\n    But please realize that I live in a capitalistic society and I need money\n    to have other people give me the things that I need to continue living\n    under the rules of said society.  So, if you feel that this piece of\n    software has helped you out enough to warrant a reward, please PayPal a\n    donation to now@disu.se\u00B9.  Thanks!  Your support won\u2019t go unnoticed!\n\n\u00B9 Send a donation:\n  https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=now@disu.se&item_name=Ame\n\n\u00A7 Reporting Bugs\n\n    Please report any bugs that you encounter to the {issue tracker}\u00B9.\n\n  \u00B9 See https://github.com/now/ame/issues\n\n\u00A7 Authors\n\n    Nikolai Weibull wrote the code, the tests, the documentation, and this\n    README.\n\n\u00A7 Licensing\n\n    Ame is free software: you may redistribute it and/or modify it under the\n    terms of the {GNU Lesser General Public License, version 3}\u00B9 or later\u00B2, as\n    published by the {Free Software Foundation}\u00B3.\n\n\u00B9 See http://disu.se/licenses/lgpl-3.0/\n\u00B2 See http://gnu.org/licenses/\n\u00B3 See http://fsf.org/\n".freeze
  s.email = ["now@disu.se".freeze]
  s.homepage = "http://disu.se/software/ame-1.0/".freeze
  s.licenses = ["LGPLv3+".freeze]
  s.rubygems_version = "3.3.7".freeze
  s.summary = "Ame provides a simple command-line interface API for Ruby\u00B9.".freeze

  s.installed_by_version = "3.3.7" if s.respond_to? :installed_by_version

  if s.respond_to? :specification_version then
    s.specification_version = 4
  end

  if s.respond_to? :add_runtime_dependency then
    s.add_runtime_dependency(%q<inventory>.freeze, ["~> 1.5"])
    s.add_development_dependency(%q<inventory-rake>.freeze, ["~> 1.6"])
    s.add_development_dependency(%q<inventory-rake-tasks-yard>.freeze, ["~> 1.4"])
    s.add_development_dependency(%q<lookout>.freeze, ["~> 3.0"])
    s.add_development_dependency(%q<lookout-rake>.freeze, ["~> 3.1"])
    s.add_development_dependency(%q<yard>.freeze, ["~> 0.8.0"])
    s.add_development_dependency(%q<yard-heuristics>.freeze, ["~> 1.2"])
  else
    s.add_dependency(%q<inventory>.freeze, ["~> 1.5"])
    s.add_dependency(%q<inventory-rake>.freeze, ["~> 1.6"])
    s.add_dependency(%q<inventory-rake-tasks-yard>.freeze, ["~> 1.4"])
    s.add_dependency(%q<lookout>.freeze, ["~> 3.0"])
    s.add_dependency(%q<lookout-rake>.freeze, ["~> 3.1"])
    s.add_dependency(%q<yard>.freeze, ["~> 0.8.0"])
    s.add_dependency(%q<yard-heuristics>.freeze, ["~> 1.2"])
  end
end
